from django.conf.urls.static import static
from django.conf.urls import include, url
from django.conf import settings

from django.views.generic import TemplateView, RedirectView

#who knew that there were so many favicons??
urlpatterns = [
    url(r'^favicon\.ico$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon.ico', permanent=False)),  
    url(r'^favicon\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon.png', permanent=False)),  
    
    url(r'^apple-touch-icon-57x57\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-57x57.png', permanent=False)),  
    url(r'^apple-touch-icon-60x60\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-60x60.png', permanent=False)),  
    url(r'^apple-touch-icon-72x72\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-72x72.png', permanent=False)),  
    url(r'^apple-touch-icon-76x76\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-76x76.png', permanent=False)),  
    url(r'^apple-touch-icon-114x114\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-114x114.png', permanent=False)),  
    url(r'^apple-touch-icon-120x120\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-120x120.png', permanent=False)),  
    url(r'^apple-touch-icon-144x144\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-144x144.png', permanent=False)),  
    url(r'^apple-touch-icon-152x152\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-152x152.png', permanent=False)),  
    url(r'^apple-touch-icon-180x180\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-180x180.png', permanent=False)),  
    url(r'^apple-touch-icon-precomposed\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'apple-touch-icon-precomposed.png', permanent=False)),  
    url(r'^favicon-16x16\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon-16x16.png', permanent=False)),  
    url(r'^favicon-32x32\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon-32x32.png', permanent=False)),  
    url(r'^favicon-96x96\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon-96x96.png', permanent=False)),  
    url(r'^favicon-160x160\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon-160x160.png', permanent=False)),  
    url(r'^favicon-192x192\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'favicon-192x192.png', permanent=False)),  
    url(r'^mstile-70x70\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'mstile-70x70.png', permanent=False)),  
    url(r'^mstile-144x144\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'mstile-144x144.png', permanent=False)),  
    url(r'^mstile-150x150\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'mstile-150x150.png', permanent=False)),  
    url(r'^mstile-310x150\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'mstile-310x150.png', permanent=False)),  
    url(r'^mstile-310x310\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'mstile-310x310.png', permanent=False)),  

    url(r'^social-icon\.png$', RedirectView.as_view(url=settings.STATIC_URL + 'social-icon.png', permanent=False)),  
    
]


from django.conf.urls import patterns, url, include

from .views import *

urlpatterns = patterns('',
   	
   	# url( (r'^%s/record/(?P<access_key>.*)\.jpeg$'%settings.EMAIL_DOMAIN), EmailRecordView.as_view(), name='email_record_view'),
   	# url( (r'^%s/view/(?P<access_key>.*)/$'%settings.EMAIL_DOMAIN), EmailOnlineView.as_view(), name='email_online_view'),
   	# url( (r'^%s/rendered/(?P<access_key>.*)/$'%settings.EMAIL_DOMAIN), EmailRenderedView.as_view(), name='email_rendered_view'),
   	# url( (r'^%s/settings/(?P<access_key>.*)/$'%settings.EMAIL_DOMAIN), EmailSettingsView.as_view(), name='email_settings_view'),

)


from django.template import Library
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.safestring import mark_safe
from django.core.exceptions import ImproperlyConfigured
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()


@register.assignment_tag()
def get_page_by_slug(slug=None):
   
    output = []
    app_label = settings.PAGE_MODEL.split('.')[0]
    object_name = settings.PAGE_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    
    try:
        item = model.objects.get(slug=slug)
    except:
        item = None

    return item


@register.assignment_tag()
def get_children_by_slug(slug=None, require_published=True):
   
    output = []
    app_label = settings.PAGE_MODEL.split('.')[0]
    object_name = settings.PAGE_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    
    try:
        item = model.objects.get(slug=slug)
        return item.get_children(require_published)
    except:
        return []  



from django.conf import settings

from django.core.paginator import Paginator, InvalidPage
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, Http404, HttpResponseRedirect, \
    HttpResponseForbidden, HttpResponsePermanentRedirect
from django.template import loader, Context, RequestContext, Template
from django.template.response import SimpleTemplateResponse
from django.template.response import TemplateResponse
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.views.generic import TemplateView
from django.views.generic.detail import SingleObjectMixin
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model



from .decorators import admins_skip_cache, users_skip_cache

from carbon.utils.template import get_template_by_pk_or_slug

class NonAdminCachableView(object):

    @method_decorator(admins_skip_cache)
    def dispatch(self, *args, **kwargs):
        return super(NonAdminCachableView, self).dispatch(*args, **kwargs)

class NonUserCachableView(object):

    @method_decorator(users_skip_cache)
    def dispatch(self, *args, **kwargs):
        return super(NonUserCachableView, self).dispatch(*args, **kwargs) 


class ObjectTemplateResponseMixin(object):

    template_name = None
    template_engine = None
    response_class = TemplateResponse
    content_type = None
    
    def get_template_names(self):

        if self.object and self.object.template:
            return [self.object.template.slug]

        elif self.model and self.model.default_template:
            return [self.model.default_template]

        elif self.template_name is None:
            raise ImproperlyConfigured(
                "ObjectTemplateResponseMixin requires either a definition of "
                "'template_name' or an object with a .template instance'")
        else:
            return [self.template_name]

    def render_to_response(self, context, **response_kwargs):
        response_kwargs.setdefault('content_type', self.content_type)
        template_names = self.get_template_names()
        return self.response_class(
            request=self.request,
            template=template_names,
            context=context,
            using=self.template_engine,
            **response_kwargs
        )

    def get_context_data(self, **kwargs):
        context = super(ObjectTemplateResponseMixin, self).get_context_data(**kwargs)
        if hasattr(self, 'object') and self.object != None:
            context['object_verbose_name'] = self.object._meta.verbose_name
            context['object_verbose_name_plural'] = self.object._meta.verbose_name_plural
            

        return context


# class CustomTemplateResponseMixin(object):

#   def get_template_names(self):
#       if not self.template_slug:
#           raise ImproperlyConfigured( "template_slug not defined.")
        
#       return [self.template_slug]


class HasSiblingsView(object):

    def get_siblings(self):
        
        try:
            return self.object.get_siblings()
        except:
            return None

    def get_next_previous(self, siblings):
        return (self.object.get_next_sibling(siblings), self.object.get_previous_sibling(siblings))

    def get(self, request, *args, **kwargs):
        self.siblings = self.get_siblings()
        self.next, self.previous = self.get_next_previous(self.siblings)

        return super(HasSiblingsView, self).get(request, *args, **kwargs)
        
    def post(self, request, *args, **kwargs):

        self.siblings = self.get_siblings()
        self.next, self.previous = self.get_next_previous(self.siblings)

        return super(HasSiblingsView, self).post(request, *args, **kwargs)
    

    def get_context_data(self, **kwargs):
        context = super(HasSiblingsView, self).get_context_data(**kwargs)
        if self.siblings != None:
            context['siblings'] = self.siblings
            context['next'] = self.next
            context['previous'] = self.previous

        return context


class HasChildrenView(HasSiblingsView):
    paginate_by = 10
    paginate_orphans = 0
    paginator_class = Paginator
    allow_empty = True
    page_kwarg = 'page'

    def paginate_queryset(self, queryset, page_size):
        """
        Paginate the queryset, if needed.
        """
        paginator = self.get_paginator(
            queryset, page_size, orphans=self.paginate_orphans,
            allow_empty_first_page=self.allow_empty)
        page_kwarg = self.page_kwarg
        page = self.kwargs.get(page_kwarg) or self.request.GET.get(page_kwarg) or 1
        try:
            page_number = int(page)
        except ValueError:
            if page == 'last':
                page_number = paginator.num_pages
            else:
                raise Http404(_("Page is not 'last', nor can it be converted to an int."))
        try:
            page = paginator.page(page_number)
            return (paginator, page, page.object_list, page.has_other_pages())
        except InvalidPage as e:
            raise Http404(_('Invalid page (%(page_number)s): %(message)s') % {
                'page_number': page_number,
                'message': str(e)
            })

    def get_paginator(self, queryset, per_page, orphans=0,
                        allow_empty_first_page=True, **kwargs):
        """
        Return an instance of the paginator for this view.
        """
        return self.paginator_class(
            queryset, per_page, orphans=orphans,
            allow_empty_first_page=allow_empty_first_page, **kwargs)

    def get_children(self):
        try:
            return self.object.get_children()
        except:
            return None

    def get(self, request, *args, **kwargs):

        if not self.object:
            self.object = self.get_object()

        try:
            children = self.get_children()
            self.object_list = children
            self.children = children
        except:
             self.object_list = None 

        return super(HasChildrenView, self).get(request, *args, **kwargs)
    

    def post(self, request, *args, **kwargs):

        if not self.object:
            self.object = self.get_object()

        try:
            children = self.get_children()
            self.object_list = children
            self.children = children
        except:
             self.object_list = None 


        return super(HasChildrenView, self).post(request, *args, **kwargs)


    


        

    def get_context_data(self, **kwargs):
        context = super(HasChildrenView, self).get_context_data(**kwargs)
        if self.object_list != None:
            paginator, page, queryset, is_paginated = self.paginate_queryset(self.object_list, self.paginate_by)
            context['paginator'] = paginator
            context['page_obj'] = page
            context['is_paginated'] = is_paginated
            context['object_list'] = queryset
            context['children'] = self.children
            context['siblings'] = self.siblings
            context['next'] = self.next
            context['previous'] = self.previous

        return context



class AddressibleView(ObjectTemplateResponseMixin, SingleObjectMixin):
    object = None
    catch_404s = True
    

    def get_template(self):
        print "TEMPLATE?? %s"%(self.object.template)
        return self.object.template

    def handle(self, request):

        if not self.object:
            self.object = self.get_object()

        if self.object.temporary_redirect != None and self.object.temporary_redirect != '':
            return HttpResponseRedirect( self.object.temporary_redirect )

        if self.object.permanent_redirect != None and self.object.permanent_redirect != '':
            return HttpResponsePermanentRedirect( self.object.permanent_redirect )

    def post(self, request, *args, **kwargs):

        self.handle(request)

        return super(AddressibleView, self).post(request, *args, **kwargs)


    def get(self, request, *args, **kwargs):
        
        self.handle(request)

        return super(AddressibleView, self).get(request, *args, **kwargs)


    def get_object_query(self, queryset, path):
        return queryset.filter(path=path).select_related('template').get()

    def get_query_path(self):
        if self.kwargs.get('path'):
            path = self.kwargs.get('path', '')
        else:
            path = self.request.path

        #Make sure path starts and ends with slashes
        if not path.endswith("/"):
            path = "%s/"%path

        if not path.startswith("/"):
            path = "/%s"%path

        return path


    def get_object(self, queryset=None):
        
        if self.object:
            return self.object

        path = self.get_query_path()

        queryset = self.get_queryset()
        
        # search_path = path
        # if hasattr(self.model, 'url_domain') and self.model.url_domain:
        #     search_path = path.replace(self.model.url_domain, '')

        try:
            # print path
            # print queryset
            obj = self.get_object_query(queryset, path)
            return obj 
        except:
            if self.catch_404s:
                raise Http404(_("No %(verbose_name)s found matching the query") %
                        {'verbose_name': queryset.model._meta.verbose_name})

            return None


import inspect
import os
import re
from importlib import import_module

from django import template
from django.template import RequestContext
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.db import models
from django.shortcuts import render_to_response
from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
from django.http import Http404
from django.core import urlresolvers
from django.contrib.admindocs import utils
from django.contrib.sites.models import Site
from django.utils._os import upath
from django.utils import six
from django.utils.translation import ugettext as _

# Exclude methods starting with these strings from documentation
MODEL_METHODS_EXCLUDE = ('_', 'add_', 'delete', 'save', 'set_')

class GenericSite(object):
    domain = 'example.com'
    name = 'my site'


@staff_member_required
def custom_model_detail(request, app_label, model_name):
    if not utils.docutils_is_available:
        return missing_docutils_page(request)

    # Get the model class.
    try:
        app_mod = models.get_app(app_label)
    except ImproperlyConfigured:
        raise Http404(_("App %r not found") % app_label)
    model = None
    for m in models.get_models(app_mod):
        if m._meta.model_name == model_name:
            model = m
            break
    if model is None:
        raise Http404(_("Model %(model_name)r not found in app %(app_label)r") % {'model_name': model_name, 'app_label': app_label})

    opts = model._meta

    # Gather fields/field descriptions.
    fields = []
    fk_fields = []
    # field_hash = {}
    for field in opts.fields:
        # ForeignKey is a special case since the field will actually be a
        # descriptor that returns the other object
        if isinstance(field, models.ForeignKey):
            data_type = field.rel.to.__name__
            app_label = field.rel.to._meta.app_label
            verbose = utils.parse_rst((_("the related `%(app_label)s.%(data_type)s` object")  % {'app_label': app_label, 'data_type': data_type}), 'model', _('model:') + data_type)
            fk_fields.append(field.name)
        else:
            data_type = get_readable_field_data_type(field)
            verbose = field.verbose_name

        # field_hash[field.name] = field

        is_meta = field.name.startswith('_')

        if is_meta==False:
            fields.append({
                'name': field.name,
                'data_type': data_type,
                'verbose': verbose,
                'help_text': field.help_text,
            })

    m2m_fields = []
    # Gather many-to-many fields.
    for field in opts.many_to_many:
        data_type = field.rel.to.__name__
        app_label = field.rel.to._meta.app_label

        verbose_all = _("%(verbose_name)s. All related `%(app_label)s.%(object_name)s` objects") % {'verbose_name':field.verbose_name, 'app_label': app_label, 'object_name': data_type}
        verbose_count = _("Number of related `%(app_label)s.%(object_name)s` objects") % {'app_label': app_label, 'object_name': data_type}
        
        #verbose_all = utils.parse_rst(_("%s%s"%(verbose_description, verbose)) , 'model', _('model:') + opts.model_name),

        fields.append({
            'name': "%s.all" % field.name,
            "data_type": 'List',
            'verbose': utils.parse_rst(verbose_all , 'model', _('model:') + opts.model_name),
        })
        fields.append({
            'name'      : "%s.count" % field.name,
            'data_type' : 'Integer',
            'verbose'   : utils.parse_rst(verbose_count , 'model', _('model:') + opts.model_name),
        })
        m2m_fields.append(field.name)
        

    # Gather related objects
    for rel in opts.get_all_related_objects() + opts.get_all_related_many_to_many_objects():
        verbose = _("related `%(app_label)s.%(object_name)s` objects") % {'app_label': rel.opts.app_label, 'object_name': rel.opts.object_name}
        accessor = rel.get_accessor_name()
        fields.append({
            'name'      : "%s.all" % accessor,
            'data_type' : 'List',
            'verbose'   : utils.parse_rst(_("All %s") % verbose , 'model', _('model:') + opts.model_name),
        })
        fields.append({
            'name'      : "%s.count" % accessor,
            'data_type' : 'Integer',
            'verbose'   : utils.parse_rst(_("Number of %s") % verbose , 'model', _('model:') + opts.model_name),
        })
        m2m_fields.append(accessor)
        

    # Gather model methods.
    for func_name, func in model.__dict__.items():
        
        if (inspect.isfunction(func) and len(inspect.getargspec(func)[0]) == 1):
            try:
                for exclude in MODEL_METHODS_EXCLUDE:
                    if func_name.startswith(exclude):
                        raise StopIteration
            except StopIteration:
                continue
            verbose = func.__doc__
            if verbose:
                verbose = utils.parse_rst(utils.trim_docstring(verbose), 'model', _('model:') + opts.model_name)
            else:
                verbose = ''
            
            data_type = get_return_data_type(func_name).strip()
            if data_type == '':
                data_type = 'Function'

            fields.append({
                'name': func_name,
                'data_type': data_type,
                'verbose': verbose,
            })
        else:
            
            verbose = func.__doc__

            ignore_list = ['DoesNotExist', 'MultipleObjectsReturned', 'objects', 'unique_together', ]

            is_fk = func_name in fk_fields
            is_m2m = func_name in m2m_fields
            is_set = func_name.endswith('_set')
            is_display = func_name.endswith('_display')
            is_uppercase = func_name.isupper()
            is_meta = func_name.startswith('_')
            is_ignore = func_name in ignore_list
            is_static_method = False if not verbose else 'staticmethod' in verbose.lower()

            add_item = is_fk == False \
                and is_set == False \
                and is_display == False \
                and is_uppercase == False \
                and is_meta == False \
                and is_ignore == False \
                and is_m2m == False \
                and is_static_method == False

            if add_item:
                
                if verbose:
                    verbose = utils.parse_rst(utils.trim_docstring(verbose), 'model', _('model:') + opts.model_name)
                else:
                    verbose = ''
                data_type = get_return_data_type(func_name).strip()
                if data_type == '':
                    data_type = 'Property'
            
                fields.append({
                    'name': func_name,
                    'data_type': data_type,
                    'verbose': verbose,                    
                })


    
    return render_to_response('admin_doc/model_detail.html', {
        'root_path': urlresolvers.reverse('admin:index'),
        'name': '%s.%s' % (opts.app_label, opts.object_name),
        # Translators: %s is an object type name
        'summary': _("Attributes on %s objects") % opts.object_name,
        'description': utils.parse_rst(model.__doc__ , 'model', _('model:') + opts.model_name),
        'fields': fields,
    }, context_instance=RequestContext(request))


####################
# Helper functions #
####################

def missing_docutils_page(request):
    """Display an error message for people without docutils"""
    return render_to_response('admin_doc/missing_docutils.html')

def get_return_data_type(func_name):
    """Return a somewhat-helpful data type given a function name"""
    if func_name.startswith('get_'):
        if func_name.endswith('_list'):
            return 'List'
        elif func_name.endswith('_count'):
            return 'Integer'
    return ''

def get_readable_field_data_type(field):
    """Returns the description for a given field type, if it exists,
    Fields' descriptions can contain format strings, which will be interpolated
    against the values of field.__dict__ before being output."""

    return field.description % field.__dict__


named_group_matcher = re.compile(r'\(\?P(<\w+>).+?\)')
non_named_group_matcher = re.compile(r'\(.*?\)')



import os
import time
import traceback
from django.utils import timezone

from django.db import models
from django.db.models.signals import pre_delete
from django.conf import settings
from django.core.urlresolvers import reverse
from django.dispatch import receiver
from django.utils.deconstruct import deconstructible
from django.utils.module_loading import import_string
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now
from django.utils.functional import cached_property
from django.core.files.storage import get_storage_class

import boto
from boto.s3.connection import S3Connection, Bucket, Key

from imagekit.models import ImageSpecField
from imagekit.processors import ResizeToFill, ResizeToFit

from .abstract import *
from .content import HasImageAtom


#HELPER FUNCTIONS

def is_incremented_file(original, updated):
    """
    Return True if files are incremented versions of eachother: avatar.png, avatar-1.png, avatar-436.png
    """
    try:
        original_path_pieces = original.split(".")
        updated_path_pieces = updated.split(".")

        original_path_start = original_path_pieces[0].split("-")
        updated_path_start = updated_path_pieces[0].split("-")
        # print 'original_path_start: %s updated_path_start: %s'%(original_path_start, updated_path_start)

        are_same_start_path = original_path_start[0] == updated_path_start[0]
        # print 'are_same_start_path? %s == %s? %s'%(original_path_start[0], updated_path_start[0], are_same_start_path)
        if are_same_start_path==False:
            return False

        original_is_integer = True if len(original_path_start)==1 else isinstance( int(original_path_start[1]), int )
        updated_is_integer = True if len(updated_path_start)==1 else isinstance( int(updated_path_start[1]), int )
        increments_are_integers = original_is_integer and updated_is_integer

        # print 'increments_are_integers? %s , %s? %s'%(original_is_integer, updated_is_integer, increments_are_integers)
        if increments_are_integers==False:
            return False

        return True

    except:
        return False

def _media_file_name( instance, filename, file_attribute_name, folder, model_name ):
    
    file, extension = os.path.splitext( filename )
    media_file = getattr(instance, file_attribute_name)

    if instance.clean_filename_on_upload:
        
        filename     = "%s%s"%(slugify(file[:245]), extension)
        filename     = filename.lower()

    full_path = '/'.join( [ folder, filename ] )
    exists = media_file.storage.exists(full_path)

    media_model = get_model(model_name.split('.')[0], model_name.split('.')[1])
    
    #If we are changing the image, delete the previous image:
    if instance.pk:
        current_instance = media_model.objects.get(pk=instance.pk)
        current_path = str(current_instance.image)

        if full_path != current_path:
            if is_incremented_file(full_path, current_path) == False:
                # print 'Delete previous media: %s'%(current_path)
                current_media_file = getattr(current_instance, file_attribute_name)
                current_media_file.storage.delete(current_path)

    
    #Handle case if file of the same name already exists
    if exists:

        #If another object in this model is using that path, then don't allow this instance to overrride it
        kwargs = {
            file_attribute_name:full_path
        }
        path_sharing_objects = media_model.objects.filter(**kwargs)
        path_sharing_objects_that_arent_this = [object for object in path_sharing_objects if object != instance]
        exists_in_separate_object = len(path_sharing_objects_that_arent_this) > 0
        

        if exists_in_separate_object==False and instance.allow_overwrite==True:
            # print 'Delete media with same name: %s'%(full_path)
            media_file.storage.delete(full_path)
        else:
            counter = 1
            starting_path = full_path
            while exists == True:
                counter += 1
                full_path = starting_path.replace(extension, '-%s%s'%(counter, extension))
                exists = media_file.storage.exists(full_path)
            # print 'Increment filename to get unique: %s'%(full_path)

    

    return full_path        


def image_file_name( instance, filename ):
    
    subfolder = (instance.__class__.__name__).lower()
    full_path = _media_file_name(instance, filename, 'image', subfolder, settings.IMAGE_MODEL)
    return full_path

def media_file_name( instance, filename ):
    
    subfolder = (instance.__class__.__name__).lower()
    full_path = _media_file_name(instance, filename, 'file', subfolder, settings.MEDIA_MODEL)
    return full_path    

def title_file_name( instance, filename ):
    
    return image_file_name(instance, filename)


def clean_path(path):
    file, extension = os.path.splitext( path )

    split = file.split('/')
    file_name = split[-1]
    file_name = file_name.replace("_", "-")
    words = file_name.split("-")
    
    #Don't include auto-generated number from above, if included in filename
    last_word = words[-1]
    is_auto_number = last_word.isdigit() and len(last_word)==10
    if is_auto_number:
        words = words[0:-1]

    combined = ' '.join(words).title()
    
    return combined



def displaybytes(bytes):
    if bytes < 1024:
        return "%sB"%(bytes)
    kb = (bytes / 1024)
    if kb < 1024:
        return '%sKB'%(kb)

    mb = (kb / 1024)
    if mb < 1024:
        return '%sMB'%(mb)

    gb = (mb / 1024)
    return '%sGB'%(gb)



def get_storage(type):
    if type=='BaseImage':
        storage = import_string(settings.IMAGE_STORAGE)()
    elif type=='BaseSecureImage':
        storage = import_string(settings.SECURE_IMAGE_STORAGE)()
    elif type=='BaseMedia':
        storage = import_string(settings.MEDIA_STORAGE)()
    elif type=='BaseSecureMedia':
        storage = import_string(settings.SECURE_MEDIA_STORAGE)()
    
    return storage




class RichContentAtom(models.Model):
    """
    This class requires a title, a file, an image, and an admin_note
    Versionable, Addressible + image and file
    """

    help = {
        'credit': "Credit",
        'caption': "Caption",
        'clean_filename_on_upload':"This removes spaces, special characters, and \
            capitalization from the file name for more consistent naming.",
        'allow_overwrite':"Allow file to write over an existing file if the name \
            is the same. If not, we'll automatically add a numerical suffix to \
            ensure file doesn't override existing files.",
        'size':'File size in bytes'
    }
    
    credit = models.CharField(_("Credit"), max_length=255, blank=True,
        help_text=help['credit'])
    caption = models.TextField(_("Caption"), blank = True,
        help_text=help['caption'] )
    clean_filename_on_upload = models.BooleanField( 
        _("Clean filename on upload"), default = True, 
        help_text=help['clean_filename_on_upload'] )
    allow_overwrite = models.BooleanField( 
        _("Allow Overwrite"), default = True, 
        help_text=help['allow_overwrite'] )

    
    size = models.BigIntegerField(null=True, blank=True, help_text=help['size'])
    display_size = models.CharField(_("Display Size"), max_length=255, blank=True, null=True)  
    file_modified_date = models.DateTimeField(_('File Modified Date'), blank=True, null=True)

    



    def save(self, *args, **kwargs):

        
        #Use filename for title if not specified.
        if self.file and not self.title:
            self.title = clean_path(self.file.url)

        if self.file:
            self.size = self.file.size
            self.display_size = displaybytes(self.file.size)
        else:
            self.size = None
            self.display_size = None

        file_changed = False
        if self.pk is not None:
            original = type(self).objects.get(pk=self.pk)
            if original.file != self.file:
                file_changed = True

        if file_changed or not self.file_modified_date:
            self.file_modified_date = timezone.now()

        super(RichContentAtom, self).save(*args, **kwargs)

    def __unicode__(self):
        if self.title:
            return ("%s :: %s")%(self.__class__.__name__, self.title)
        elif self.caption or self.credit:
            return ("%s :: %s %s")%(self.__class__.__name__, self.caption, self.credit)
        else:
            return ("%s :: %s")%(self.__class__.__name__, self.pk)

    @cached_property
    def cache_source_path(self):
        """
        NOTE -- this function assumes we're using the default imagekit namer function
        """
    
        source_filename = getattr(self.file, 'name', None)

        if source_filename is None or os.path.isabs(source_filename):
            # Generally, we put the file right in the cache file directory.
            dir = settings.IMAGEKIT_CACHEFILE_DIR
        else:
            # For source files with relative names (like Django media files),
            # use the source's name to create the new filename.
            dir = os.path.join(settings.IMAGEKIT_CACHEFILE_DIR,
                               os.path.splitext(source_filename)[0])
        return dir

    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "title__icontains", "credit__icontains","caption__icontains",'admin_note__icontains')

    class Meta:
        abstract = True        

class BaseSecureAtom(models.Model):

    @staticmethod
    def make_file_private(file):        
        try:
            if file:
                key_name = "%s/%s"%(settings.AWS_MEDIA_FOLDER, file.name)
                BaseSecureAtom.make_private(settings.AWS_STORAGE_BUCKET_NAME_MEDIA_SECURE, key_name)                
        except:
            print 'Error making file private: %s'%(traceback.format_exc())

    @staticmethod
    def make_private(bucket_name, key_name):        
        try:
            connection = boto.s3.connection.S3Connection(
                settings.AWS_ACCESS_KEY_ID,
                settings.AWS_SECRET_ACCESS_KEY)
            bucket = connection.create_bucket(bucket_name)
            key = boto.s3.key.Key(bucket)
            key.key = key_name
            key.set_acl('private')
        except:
            print 'Error making private: %s'%(traceback.format_exc())

    @staticmethod
    def generate_authorized_link(file, duration_seconds):

        key_name = "%s/%s"%(settings.AWS_MEDIA_FOLDER, file.name)
        return BaseSecureAtom.generate_link(settings.AWS_STORAGE_BUCKET_NAME_MEDIA_SECURE, key_name, duration_seconds)  

    @staticmethod
    def generate_link(bucket, key_name, duration_seconds):

        connection = boto.s3.connection.S3Connection(
            settings.AWS_ACCESS_KEY_ID,
            settings.AWS_SECRET_ACCESS_KEY,
            is_secure=True)
        
        return connection.generate_url(duration_seconds, 'GET',
            bucket=bucket,
            key=key_name,
            force_http=True)

    class Meta:
        abstract = True


class BaseImageMolecule( RichContentAtom, VersionableAtom, AddressibleAtom ):

    help = {
        'alt':"Alt text (very important for SEO)",
        'use_png':"Render image as png instead of jpg when possible"
    }

   
    alt = models.CharField(_("Alt Text"), max_length=255, blank=True,
        help_text=help['alt'])

    # -- Variations
    thumbnail_jpg = ImageSpecField( source='image', format='JPEG', 
        processors=[ResizeToFit(settings.IMAGE_THUMBNAIL_WIDTH, 
        settings.IMAGE_THUMBNAIL_HEIGHT)], 
        options={'quality': settings.IMAGE_THUMBNAIL_QUALITY})
    thumbnail_png = ImageSpecField( source='image', format='PNG', 
        processors=[ResizeToFit(settings.IMAGE_THUMBNAIL_WIDTH, 
        settings.IMAGE_THUMBNAIL_HEIGHT)],
        options={'quality': settings.IMAGE_THUMBNAIL_QUALITY})

    use_png = models.BooleanField( default = False, 
        verbose_name='Use .PNG (instead of .JPG)', help_text=help['use_png'])

    image_width = models.IntegerField(null=True, blank=True)
    image_height = models.IntegerField(null=True, blank=True)
    

    class Meta:
        abstract = True

    def get_alt(self):
        if self.alt:
            return self.alt
        return self.title

    @cached_property
    def get_format(self):
        if self.use_png:
            return 'PNG'
        return 'JPEG'
    
    @cached_property
    def image_url(self):
        try:
            return self.image.url
        except:
            return None

    @cached_property
    def image_width(self):
        try:
            return self.image.width
        except:
            return None

    @cached_property
    def image_height(self):
        try:
            return self.image.height
        except:
            return None

    @cached_property
    def file(self):
        return self.image  

    @cached_property
    def thumbnail(self):
        if self.use_png:
            return self.thumbnail_png  
        else:
            return self.thumbnail_jpg

    @cached_property
    def thumbnail_url(self):
        try:
            return self.thumbnail.url
        except:
            return ''


    def get_variant_url(self, variant_name):
        try:
            field = getattr(self, variant_name)
            return field.url
        except Exception, err:
            print traceback.format_exc()

    def get_variant_width(self, variant_name):
        try:
            field = getattr(self, variant_name)
            return field.width
        except:
            return None  


    def get_variant_height(self, variant_name):
        try:
            field = getattr(self, variant_name)
            return field.height
        except:
            return None  
    
    def get_variant_link(self, variant_name, include_dimensions=False):
        # try:
        field = getattr(self, variant_name)
        gussied_name = variant_name.replace("_", " ").title()
        
        if variant_name=='image':
            return '<a href="%s" data-img="%s" data-alt="%s" data-credit="%s" data-caption="%s">%s (%spx x %spx)</a>'\
                %(field.url, field.url, self.get_alt(), self.credit, self.caption, 'Original Image', self.image_width, self.image_height)
        
        elif include_dimensions:
            return '<a href="%s" data-img="%s" data-alt="%s" data-credit="%s" data-caption="%s">%s (%spx x %spx)</a>'\
                %(field.url, field.url, self.get_alt(), self.credit, self.caption, gussied_name, field.width, field.height)
        else:
            field_description = ''

            print hasattr(self, 'help')
            print variant_name
            print variant_name in self.help
            print self.help
            
            if hasattr(self, 'help') and variant_name in self.help:
                field_description = ' (%s)'%self.help[variant_name]
            elif hasattr(self, 'help_text') and variant_name in self.help_text:
                field_description = ' (%s)'%self.help_text[variant_name]
            
            return '<a href="%s" data-img="%s" data-alt="%s" data-credit="%s" data-caption="%s">%s%s</a>'\
            %(field.url, field.url, self.get_alt(), self.credit, self.caption, gussied_name, field_description)
        # except:
        #     return ''

    

    def save(self, *args, **kwargs):

        
        if self.image:
            self.image_width = self.image.width
            self.image_height = self.image.height
            self.size = self.image.size
            self.display_size = displaybytes(self.image.size)
        else:
            self.image_width = None
            self.image_height = None
            self.size = None
            self.display_size = None
            

        super(BaseImageMolecule, self).save(*args, **kwargs)


    variants = ('thumbnail',)

class ImageMolecule( BaseImageMolecule ):
    help = {
        'image':"To ensure a precise color replication in image variants, make sure an sRGB color profile has been assigned to each image.",
    }


    try:
        image = models.ImageField(upload_to=image_file_name, blank=True, null=True,storage=get_storage('BaseImage'), help_text=help['image'])
    except:
        image = models.ImageField(upload_to=image_file_name, blank=True, null=True, help_text=help['image'])



    class Meta:
        abstract = True


class SecureImageMolecule( BaseImageMolecule ):
    help = {
        'image':"To ensure a precise color replication in image variants, make sure an sRGB color profile has been assigned to each image.",
    }
    default_image_link_duration = 60*60 #= 1 hour


    try:
        image = models.ImageField(upload_to=image_file_name, blank=True, null=True,storage=get_storage('BaseSecureImage'), help_text=help['image'])
    except:
        image = models.ImageField(upload_to=image_file_name, blank=True, null=True, help_text=help['image'])


    @cached_property
    def image_url(self):
        if self.image:
            return BaseSecureAtom.generate_authorized_link(self.image, self.default_image_link_duration)
        else:
            return None

    def save(self, *args, **kwargs):

        super(SecureImageMolecule, self).save(*args, **kwargs)

        if self.image:
            BaseSecureAtom.make_file_private(self.image)


    class Meta:
        abstract = True


class MediaMolecule( ImageMolecule ):

    class Meta:
        verbose_name_plural = 'media'
        abstract = True

    try:
        file = models.FileField(upload_to=media_file_name, blank=True, null=True,storage=get_storage('BaseMedia'))
    except:
        file = models.FileField(upload_to=media_file_name, blank=True, null=True)

class SecureMediaMolecule( BaseSecureAtom, RichContentAtom, VersionableAtom, AddressibleAtom ):

    
    class Meta:
        verbose_name_plural = 'secure media'
        abstract = True

    try:
        file = models.FileField(upload_to=media_file_name, blank=True, null=True,storage=get_storage('BaseSecureMedia'))
    except:
        file = models.FileField(upload_to=media_file_name, blank=True, null=True)       

    def get_secure_url(self, duration=60):
        if self.file:
            return BaseSecureAtom.generate_authorized_link(self.file, duration)
        return None

    def save(self, *args, **kwargs):

        super(SecureMediaMolecule, self).save(*args, **kwargs)

        if self.file:
            BaseSecureAtom.make_file_private(self.file)






try:
    image_model = settings.IMAGE_MODEL
    @receiver(pre_delete, sender=image_model, dispatch_uid='image_delete_signal')
    def remove_image_file_from_s3(sender, instance, using, **kwargs):
        base_remove_image_file_from_s3(sender, instance, using, settings.IMAGE_MODEL, settings.MEDIA_STORAGE)
except:
    pass

try:
    secure_image_model = settings.SECURE_IMAGE_MODEL
    @receiver(pre_delete, sender=secure_image_model, dispatch_uid='secure_image_delete_signal')
    def remove_secure_file_from_s3(sender, instance, using, **kwargs):
        base_remove_image_file_from_s3(sender, instance, using, settings.SECURE_IMAGE_MODEL, settings.SECURE_MEDIA_STORAGE)    
except:
    pass

def base_remove_image_file_from_s3(sender, instance, using, model_name, storage_bucket_path, **kwargs):
    try:
        delete_file_on_delete = settings.IMAGE_MODEL_DELETE_FILE_ON_DELETE
    except:
        delete_file_on_delete = False

    model = get_model(model_name.split('.')[0], model_name.split('.')[1])
    path_sharing_objects = model.objects.filter(image=instance.image)
    path_sharing_objects_that_arent_this = [object for object in path_sharing_objects if object != instance]
    exists_in_separate_object = len(path_sharing_objects_that_arent_this) > 0
    

    if exists_in_separate_object==False and delete_file_on_delete:

        #Delete all the corresponding cached images:
        if hasattr(instance, 'variants') and instance.variants:

            storage_bucket_class = get_storage_class(storage_bucket_path)
            storage_bucket = storage_bucket_class()
            
            connection = storage_bucket.connection
            bucket = storage_bucket.bucket
            delete_path = '%s/%s'%(storage_bucket.location, instance.cache_source_path)

            for key in bucket.list(prefix=delete_path):
                key.delete()

        try:
            instance.image.delete(save=False)  
        except:
            pass

try:
    document_model = settings.DOCUMENT_MODEL
    @receiver(pre_delete, sender=document_model, dispatch_uid='document_delete_signal')
    def remove_document_file_from_s3(sender, instance, using, **kwargs):
        base_remove_document_file_from_s3(sender, instance, using, settings.DOCUMENT_MODEL)
except:
    pass

try:
    secure_document_model = settings.SECURE_DOCUMENT_MODEL
    @receiver(pre_delete, sender=secure_document_model, dispatch_uid='secure_document_delete_signal')
    def remove_secure_file_from_s3document_file_from_s3(sender, instance, using, **kwargs):
        base_remove_document_file_from_s3(sender, instance, using, settings.SECURE_DOCUMENT_MODEL)
except:
    pass

def base_remove_document_file_from_s3(sender, instance, using, model_name, **kwargs):
    try:
        delete_file_on_delete = settings.DOCUMENT_MODEL_DELETE_FILE_ON_DELETE
    except:
        delete_file_on_delete = False

    model = get_model(model_name.split('.')[0], model_name.split('.')[1])
    path_sharing_objects = model.objects.filter(media_file=instance.media_file)
    path_sharing_objects_that_arent_this = [object for object in path_sharing_objects if object != instance]
    exists_in_separate_object = len(path_sharing_objects_that_arent_this) > 0

    if exists_in_separate_object==False and delete_file_on_delete:
        try:
            instance.media_file.delete(save=False)  
        except:
            pass            

 



from django.template import Library
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.safestring import mark_safe
from django.core.exceptions import ImproperlyConfigured
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model
    
from ..models import *

register = Library()


@register.assignment_tag()
def get_link_descendants(slug=None, item=None):
    """
    This tag gets the children and (if they're there) grandchildren of a link item.
    """
    if slug is None and item is None:
        raise ImproperlyConfigured("Either slug or item must be defined")
        


    output = []
    app_label = settings.MENU_MODEL.split('.')[0]
    object_name = settings.MENU_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    
    if not item:
        try:
            item = model.objects.get(slug=slug)
        except:
            item = None

    if slug:
        children = model.objects.filter(parent__slug=slug).order_by('order').prefetch_related('content_object')
    elif item:
        children = model.objects.filter(parent=item).order_by('order').prefetch_related('content_object')

    children = [] if item is None else [child for child in item.get_children() if child.is_published()]

    descendants = {
        'item':item,
        'children':output
    }
    for child in children:
        output.append({
            'item':child,
            'children':get_link_descendants(None, child)
        })

    return descendants



@register.assignment_tag(takes_context=True)
def get_edit_url(context, object):
    request = context['request']
    if request.user and request.user.is_authenticated() and request.user.is_staff:
        if object:
            try:
                return object.edit_item_url
            except:
                pass
        return None
    return None


#TODO -- figure out a way to make this check if user is admin before returning url
@register.filter(is_safe=True)
def edit_attribute(object):
    if object:
        try:
            return mark_safe('data-edit-url="%s"'%(object.edit_item_url))
        except:
            pass
    return None



@register.assignment_tag()
def get_js_package(slug, minified=False):
    model = get_model('core', 'JSPackage')
    try:
        item = model.objects.get(slug=slug)
    except:
        item = None

    if item:
        return item.get_url(minified)
    return ''

@register.assignment_tag()
def get_js_source(slug, minified=False):
    model = get_model('core', 'JSPackage')
    try:
        item = model.objects.get(slug=slug)
    except:
        item = None

    if item:
        if minified:
            return item.generated_file_minified
        else:
            return item.generated_file_source
    return ''   


@register.assignment_tag()
def get_css_package(slug, minified=False):
    model = get_model('core', 'CSSPackage')
    try:
        item = model.objects.get(slug=slug)
    except:
        item = None

    if item:
        return item.get_url(minified)
    return ''   

@register.assignment_tag()
def get_css_source(slug, minified=False):
    model = get_model('core', 'CSSPackage')
    try:
        item = model.objects.get(slug=slug)
    except:
        item = None

    if item:
        if minified:
            return item.generated_file_minified
        else:
            return item.generated_file_source
    return ''   



from django.conf import settings

from django.core.exceptions import PermissionDenied
from django.views.generic.detail import SingleObjectMixin


class AccessibleView(SingleObjectMixin):


    def get_object(self, queryset=None):
        
        self.object = super(AccessibleView, self).get_object(queryset)
        
        if self.object.access_allowed(self.request) == False:
            raise PermissionDenied()

        return self.object


import boto
from optparse import make_option
import os
import time
from time import mktime
from datetime import datetime

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


from carbon.utils.data.backup_mysql import backup_database as backup_database_mysql
from carbon.utils.data.backup_postgres import backup_database as backup_database_postgres
from carbon.utils.data.backup_sqlite import backup_database as backup_database_sqlite

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--bucket',
            action='store_true',
            dest='storage_bucket',
            default=None,
            help='AWS Storage Bucket'),
        make_option('--db',
            action='store_true',
            dest='database_name',
            default='default',
            help='Database to backup'),
        make_option('--prefix',
            action='store_true',
            dest='prefix',
            default='database_backup',
            help='Filename prefix'),
        make_option('--temp_dir',
            action='store_true',
            dest='temp_dir',
            default='/tmp',
            help='Temporary store directory'),
        make_option('--dest_dir',
            action='store_true',
            dest='dest_dir',
            default='database/',
            help='Subdirectory on AWS bucket to store backup'),
        make_option('--delete_old_backups',
            action='store_true',
            dest='delete_old_backups',
            default=True,
            help='Delete old back-ups in bucket'),
        make_option('--backups_days',
            action='store_true',
            dest='backups_days',
            default=30,
            help='How many days to keep old backups if delete_old_backups==True'),
    )

    def handle(self, *args, **options):

        if options['storage_bucket'] == None:
            try:
                bucket_name = settings.AWS_STORAGE_BUCKET_NAME_MEDIA_SECURE
            except:
                try:
                    bucket_name = settings.AWS_STORAGE_BUCKET_NAME
                except:
                    bucket_name = None
        else:
            bucket_name = options['storage_bucket']

        if bucket_name:
            print "Going to dump database %s into bucket %s using file prefix %s temp store to %s then store in folder %s"%(options['database_name'], bucket_name, options['prefix'], options['temp_dir'], options['dest_dir'])

            

            if options['database_name'] in settings.DATABASES:
                db = settings.DATABASES[options['database_name']]
                db_engine = db['ENGINE']
                if 'mysql' in db_engine.lower():
                    backup_database_mysql(bucket_name, options['database_name'], options['dest_dir'], options['prefix'], options['temp_dir'])

                elif 'postgresql_psycopg2' in db_engine.lower():
                    backup_database_postgres(bucket_name, options['database_name'], options['dest_dir'], options['prefix'], options['temp_dir'])

                elif 'sqlite3' in db_engine.lower():
                    backup_database_sqlite(bucket_name, options['database_name'], options['dest_dir'], options['prefix'], options['temp_dir'])
                    
            else:
                print "ERROR: No database found with name %s in settings.DATABASES"%(options['database_name'])

            if options['delete_old_backups']:
                conn = boto.connect_s3(settings.AWS_ACCESS_KEY_ID, settings.AWS_SECRET_ACCESS_KEY)
                bucket = conn.get_bucket(bucket_name)
                max_days = int(options['backups_days'])
                now = datetime.now()
                if max_days > 0:                    
                    for key in bucket:                                          
                        if options['prefix'] in key.name:

                            try:
                                #Unfortunately S3/boto doesn't provide a created date and the modified date updates anytime you look at it
                                #So we're going off the filename
                                created = time.strptime(key.name, options['dest_dir']+options['prefix']+'_%Y-%m-%d_%H-%M.sql.zip')
                                
                                #convert to datetime
                                created_dt = datetime.fromtimestamp(mktime(created))
                                diff_dy = now - created_dt
                                if diff_dy.days > max_days:
                                    print "Delting %s, which is %s days old"%(key.name, diff_dy.days)
                                    key.delete()
                            except:
                                pass
                        






import re
import urllib
from unidecode import unidecode
from django import template
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.template import Library
from django.template.defaultfilters import slugify
from django.utils.html import strip_tags


from ..models import *

register = Library()



#HELPER FUNCTIONS:

@register.assignment_tag
def getAttribute(the_object, attribute_name):
    # Try to fetch from the object, and if it's not found return None.
    # print 'get %s from %s'%(attribute_name, the_object)

    try:
        attr = the_object[attribute_name]
    except:
        attr = getattr(the_object, attribute_name, None)
    return attr

@register.simple_tag(takes_context=True)
def url_add_query(context, **kwargs):
    request = context.get('request')

    get = request.GET.copy()
    get.update(kwargs)

    path = '%s?' % request.path
    for query, val in get.items():
        path += '%s=%s&' % (query, val)


    return path[:-1]

@register.assignment_tag(takes_context=True)
def get_query_string(context):
    request = context.get('request')

    get = request.GET.copy()
    
    path = '?'
    for query, val in get.items():
        path += '%s=%s&' % (query, val)


    return path[:-1]


@register.filter
def tostring(object):
    return str(object)

@register.filter
def slugify(value):
    return slugify(unidecode(value))

def insert_ellipse(lst, ellipse=0):
    '''Insert ellipse where it's not sequential

    An ellipse is represented by @ellipse.

    E.g.:
        [1, 2, 5] -> [1, 2, 0, 5]
        [1, 2, 4] -> [1, 2, 3, 4]

    The last example is that if only one step is missing, just add it instead
    of an ellipse.
    '''

    if len(lst) <= 1:
        return list(lst[:])
    gap = lst[1] - lst[0]
    if gap <= 1:
        return [lst[0]] + insert_ellipse(lst[1:], ellipse)
    elif gap == 2:
        return [lst[0], lst[0] + 1] + insert_ellipse(lst[1:], ellipse)
    else:
        return [lst[0], ellipse] + insert_ellipse(lst[1:], ellipse)

@register.assignment_tag
def pick_pages(num_pages, current):
    '''pick pages to display in the pager

    It's like: 1 ... (x-2) (x-1) (x) (x+1) (x+2) ... N.
    '''

    ret = []

    for p in range(current - 2, current):
        if p >= 1:
            ret.append(p)
    ret.append(current)
    for p in range(current + 1, current + 3):
        if p <= num_pages:
            ret.append(p)

    if ret[0] != 1:
        ret.insert(0, 1)
    if ret[-1] != num_pages:
        ret.append(num_pages)
    ret = insert_ellipse(ret)

    return ret


@register.assignment_tag()
def url_to_edit_object(object):
    try:
        object_type = type(object).__name__
        if object_type == 'SearchResult':
            object = object.object
        
        url = reverse('admin:%s_%s_change' %(object._meta.app_label,  object._meta.model_name),  args=[object.id] )
        return url
    except:
        return None

@register.assignment_tag()
def get_model_label(object):
    try:
        object_type = type(object).__name__
        object_label = object.__unicode__()

        if object_type == 'SearchResult':
            object = object.object
            object_type = type(object).__name__
            object_label = object.__unicode__()

        return "%s %s"%(object_type.upper(), object_label)
        
    except:
        return None        




def is_path_encrypted(path):
    try:
        for SSL_PATH in settings.SSL_PATHS:
            pattern = re.compile(SSL_PATH)
            match = pattern.match(path)
            if match:
                #print 'Matched %s to pattern %s'%(path, SSL_PATH)
                return True
    except:
        return False



@register.assignment_tag(takes_context=True)
def get_canonical_url(context):

    try:
        request = context['request']
    except:
        #No request
        return None
        
    current_site = Site.objects.get_current()
    path = request.get_full_path()

    encrypt_path = is_path_encrypted(path)

    if encrypt_path:
        url  = 'https://%s%s'%(current_site.domain, path)
    else:
        url  = 'http://%s%s'%(current_site.domain, path)

    return url


@register.assignment_tag(takes_context=True)
def get_site_url(context):
    try:
        request = context['request']
    except:
        #No request
        return None
        
    request = context['request']
    full_path = ('http', ('', 's')[request.is_secure()], '://', request.META['HTTP_HOST'])
    domain = ''.join(full_path)
    
    return domain

@register.filter
def data_verbose(boundField):
    if boundField:
        value = boundField.value()
        field = boundField.field    
        return hasattr(field, 'choices') and dict(field.choices).get(value,'') or value
    return None

@register.assignment_tag()
def get_sorted(list, attribute, reverse=False):
    return sorted(list, key=lambda x: getattr(x, attribute), reverse=reverse)


import re
import urllib
from unidecode import unidecode
from django import forms
from django import template
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.template import Library


try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from ..models import *


# from carbon.compounds.blog.forms import BlogCommentForm

register = Library()


@register.assignment_tag(takes_context=True)
def can_user_edit_comment(context, comment):
    request = context['request']
    return request.user == comment.user

@register.assignment_tag()
def get_blog_article_comments(article):
    blog_comment_model = get_model(settings.BLOG_COMMENT_MODEL.split('.')[0], settings.BLOG_COMMENT_MODEL.split('.')[1])
    return blog_comment_model.get_comments_for_article(article)


@register.assignment_tag(takes_context=True)
def get_blog_comment_form(context, article, form):
    
    form_class = type(form)
    return form_class.create_submit_form(context, form, article)


@register.assignment_tag(takes_context=True)
def get_blog_comment_reply_form(context, comment, form):

    form_class = type(form)
    return form_class.create_reply_form(context, form, comment)
    

@register.assignment_tag(takes_context=True)
def get_blog_comment_update_form(context, comment, form):

    form_class = type(form)
    return form_class.create_update_form(context, form, comment)

@register.assignment_tag(takes_context=True)
def get_blog_comment_delete_form(context, comment, form):

    form_class = type(form)
    return form_class.create_delete_form(context, form, comment)

@register.assignment_tag(takes_context=True)
def get_blog_comment_upvote_form(context, comment, form):
    form_class = type(form)
    return form_class.create_upvote_form(context, form, comment)      

@register.assignment_tag(takes_context=True)
def get_blog_comment_downvote_form(context, comment, form):
    form_class = type(form)
    return form_class.create_downvote_form(context, form, comment)            

@register.assignment_tag(takes_context=True)
def get_blog_comment_flag_form(context, comment, form):
    form_class = type(form)
    return form_class.create_flag_form(context, form, comment)            



from django import forms
from django.utils.encoding import force_text

from django.forms.utils import flatatt, to_current_timezone
from django.forms.widgets import TextInput as BaseTextInput
from django.forms.widgets import Textarea as BaseTextarea
from django.forms.widgets import CheckboxInput as BaseCheckboxInput
from django.forms.widgets import Select as BaseSelect
from django.forms.widgets import SelectMultiple as BaseSelectMultiple
from django.forms.widgets import ClearableFileInput as BaseClearableFileInput


from django.template import loader
from django.template import Context, Template
from django.utils.html import conditional_escape, format_html



class BaseFormInput(object):
	has_custom_render = True
	model_field = None
	bound_field = None
	file_template = 'form/partials/base_field_container.html'

	def __init__(self, model_field, attrs=None):
		self.model_field = model_field
		if attrs is not None:
			self.attrs = attrs.copy()
		else:
			self.attrs = {}

	def render_input(self, context_data, context=None):
		if context==None:
			context = Context(context_data)


		context['model_field'] = self.model_field
		context['field'] = self.bound_field

		template = loader.get_template(self.file_template)

		return template.render(context)

class BaseMultipleChoiceFormInput(BaseFormInput):

	def __init__(self, model_field, attrs=None, choices=()):
		super(BaseMultipleChoiceFormInput, self).__init__(model_field, attrs)

		self.choices = list(choices)

class SingleLineText(BaseFormInput, BaseTextInput):
	input_type = 'text'


	def render(self, field, context=None, attrs=None):
		self.bound_field = field

		value = field.value()
		name = field.html_name
		if value is None:
			value = ''
		final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
		if value != '':
			# Only add the 'value' attribute if a value is non-empty.
			final_attrs['value'] = force_text(self._format_value(value))        
		
		
		return self.render_input(final_attrs, context)

class MultiLineText(BaseFormInput, BaseTextarea):

	def render(self, field, context=None, attrs=None):
		self.bound_field = field
		value = field.value()
		name = field.html_name

		if value is None:
			value = ''
		final_attrs = self.build_attrs(attrs, name=name)
		return self.render_input(final_attrs, context)

class Checkbox(BaseFormInput, BaseCheckboxInput):
	
	def check_test(self, v):
		return not (v is False or v is None or v == '')

	def render(self, field, context=None, attrs=None):
		self.bound_field = field
		value = field.value()
		name = field.html_name

		final_attrs = self.build_attrs(attrs, type='checkbox', name=name)
		if self.check_test(value):
			final_attrs['checked'] = 'checked'
		if not (value is True or value is False or value is None or value == ''):
			# Only add the 'value' attribute if a value is non-empty.
			final_attrs['value'] = force_text(value)
		return self.render_input(final_attrs, context)

class Select(BaseMultipleChoiceFormInput, BaseSelect):
	def render(self, field, context=None, attrs=None):
		self.bound_field = field
		value = field.value()
		name = field.html_name

		final_attrs = self.build_attrs(attrs, name=name)

		return self.render_input(final_attrs, context)

class SelectMultiple(BaseMultipleChoiceFormInput, BaseSelectMultiple):
	def render(self, field, context=None, attrs=None):
		self.bound_field = field
		value = field.value()
		name = field.html_name

		final_attrs = self.build_attrs(attrs, name=name)

		return self.render_input(final_attrs, context)

	# def decompress(self, value):
	# 	print "DECOMPRESS VALUE: %s"%(value)

	# 	super(SelectMultiple, self).decompress(value)
		

class ClearableFile(BaseFormInput, BaseClearableFileInput):
	def render(self, field, context=None, attrs=None):
		self.bound_field = field
		value = field.value()
		name = field.html_name

		final_attrs = self.build_attrs(attrs, name=name)

		return self.render_input(final_attrs, context)   


class MultipleChoiceField(forms.MultipleChoiceField):
		
	def valid_value(self, value):
		

		if isinstance(value, (basestring)):
			try:
				value_as_list = eval(value)
			except:
				value_as_list = value
		else:
			value_as_list = value
		
		choices_values = dict(self.choices).keys()
		choices_values = [choice.strip() for choice in choices_values]
		
		if isinstance(value_as_list, (list)) == False:
			value_as_list = [value_as_list]
			# print 'render to list %s'%(value_as_list)


		for value_item in value_as_list:
			found_match = False
			
			for choice in choices_values:
				if choice == value_item.strip():
					found_match = True

			if found_match == False:
				print 'WARNING: Value %s-%s not in choices list %s'%(value, value_item, choices_values)
				return False

		return True

	def prepare_value(self, value):
		#suss out the field value
		if value and (len(value)>0):
			first_element = value[0]
			if isinstance(first_element, (list)):
				value = first_element

		return value






from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.functional import cached_property

from .abstract import *

class AccessAtom(models.Model):

	help = {
		'access':"Access level",
	}

	OWNER = 10
	WRITE = 20
	READ = 30
	ACCESS_CHOICES = (
		(OWNER, _("Owner")),
		(WRITE, _("Can Edit")),
		(READ, _("Can View")),
	)

	access = models.IntegerField(choices=ACCESS_CHOICES, 
		help_text=help['access'])

	class Meta:
		abstract = True


class AccessKeyAtom(models.Model):

	access_key = models.CharField(_("key"), max_length=50, blank=True, null=True, unique=True)

	def save(self, *args, **kwargs):

		if not self.access_key:
			self.access_key = uuid.uuid1().hex 

		super(AccessKeyAtom, self).save(*args, **kwargs)
		

	class Meta:
		abstract = True

# -- Level 4
class AccessibleAtom(models.Model):

	help = {
		'password':"Password to use if access_restriction is set to Password",
		'require_registered_user' : "Require logged in user" 
	}

	# OPEN = 1
	# REGISTERED_USERS = 2
	# PASSWORD = 3
	# USER_WHITELIST = 4
	# USER_BLACKLIST = 5
	# CUSTOM = 6
	# ACCESS_CHOICES = (
	#     (OPEN, _("Open")),
	#     (REGISTERED_USERS, _("Registered Users")),
	#     (PASSWORD, _("Password")),
	#     (USER_WHITELIST, _("User Whitelist")),
	#     (USER_BLACKLIST, _("User Blacklist")),
	#     (CUSTOM, _("Custom")),
	# )


	require_registered_user = models.BooleanField( _("Required Registered Users"), 
		default = False, help_text=help['require_registered_user'])

	password = models.CharField(_('Encrypted Password'), max_length=255, 
		null=True, blank=True, help_text=help['password'])

	#THOUGHT: Not sure if I love this design
	user_whitelist = models.ManyToManyField(settings.AUTH_USER_MODEL, 
		blank=True, null=True, related_name='%(app_label)s_%(class)s_whitelist_user')

	user_blacklist = models.ManyToManyField(settings.AUTH_USER_MODEL, 
		blank=True, null=True, related_name='%(app_label)s_%(class)s_blacklist_user')

	groups_whitelist = models.ManyToManyField('auth.Group', 
		blank=True, null=True, related_name='%(app_label)s_%(class)s_whitelist_groups')

	groups_blacklist = models.ManyToManyField('auth.Group', 
		blank=True, null=True, related_name='%(app_label)s_%(class)s_blacklist_groups')

	

	class Meta:
		abstract = True

	def access_allowed(self, request):
		
		if self.require_registered_user: 
			if not request.user or not request.user.is_authenticated():
				return False

		if len(self.object.groups_whitelist.all()) > 0:
			#make sure user is allowed
			#TODO
			return False

		if len(self.object.groups_blacklist.all()) > 0:
			#make sure user is allowed 
			#TODO
			return False       

		if len(self.user_whitelist.all()) > 0:
			if not request.user or not request.user.is_authenticated() \
			or not request.user in user_whitelist:
				return False

		if len(self.user_blacklist.all()) > 0:
			if not request.user or not request.user.is_authenticated() \
			or request.user in user_blacklist:
				return False

		#TODO -- password

		
		return True

	def custom_access_allowed(self, request):
		#OVERRIDE IN SUBCLASS
		return True


from django.contrib import admin
from django.core.urlresolvers import reverse

from carbon.atoms.admin.content import *
from carbon.atoms.admin.taxonomy import *

from reversion.admin import VersionAdmin

from .forms import PageAdminForm, PageContentBlockAdminForm, GlobalContentBlockAdminForm

from django_inline_wrestler.admin import TabularInlineOrderable



class PageAdmin(VersionAdmin, HierarchicalContentAdmin):
    form = PageAdminForm

    autocomplete_lookup_fields = HierarchicalContentAdmin.autocomplete_lookup_fields
    
    m2m_fields_list = list(autocomplete_lookup_fields['m2m'])
    # m2m_fields_list.insert(0, 'tags')
    autocomplete_lookup_fields['m2m'] = tuple(m2m_fields_list)

    raw_id_fields = HierarchicalContentAdmin.raw_id_fields
    raw_id_fields_list = list(raw_id_fields)
    # raw_id_fields_list.insert(0, 'tags')
    raw_id_fields = tuple(raw_id_fields_list)

    core_fields = HierarchicalContentAdmin.core_fields
    core_fields_list = list(core_fields)
    # core_fields_list.insert(5, 'tags')
    core_fields = tuple(core_fields_list)

    path_fields = BaseContentAdmin.path_fields
    publication_fields = BaseContentAdmin.publication_fields
    seo_fields = BaseContentAdmin.seo_fields
    social_fields = BaseContentAdmin.social_fields
    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

    # list_display = ( "parent", "admin_hierarchy", "hierarchy", "path",  "title", "publication_status",)

class PageContentBlockInline(admin.StackedInline):

    form = PageContentBlockAdminForm
    sortable_field_name = "order"
    extra = 0
    
    classes = ('grp-collapse grp-open',)
    inline_classes = ('grp-collapse grp-open',)

    meta_fields = BaseVersionableAdmin.meta_fields

    
    core_fields = (
        ('order','title','slug'),
        ('content',),
    )
    additional_fields = (
        ('synopsis'),
        ('publication_status', 'publication_date'),
        ('publish_on_date', 'expire_on_date')
    )

    fieldsets = (
        ("Content", {
            'fields': core_fields
        }),
        ("Additional Properties", {
            'fields': additional_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
    )

    readonly_fields = BaseVersionableAdmin.readonly_fields


class PageTagAdmin(BaseTagAdmin):
    pass


class MenuItemInline(TabularInlineOrderable):
    #model = MenuItem    

    autocomplete_lookup_fields = {
        'generic': [['content_type', 'object_id']],
        'fk': [],
    }
    fields = ('order','title','content_type', 'object_id','path_override','edit_item',)
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('path','edit_item',)
    extra = 0







class GlobalContentBlockAdmin(BaseVersionableTitleAdmin):
    #VersionableAtom, TitleAtom, ContentAtom, PublishableAtom

    form = GlobalContentBlockAdminForm

    
    core_fields = BaseVersionableTitleAdmin.core_fields
    core_fields_list = list(core_fields)
    core_fields_list.append('content')
    core_fields = tuple(core_fields_list)

    meta_fields = BaseVersionableTitleAdmin.meta_fields

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
       
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )




# admin.site.register(Page, PageAdmin)
# admin.site.register(PageTag, PageTagAdmin)



from django.conf import settings
from django.contrib.sitemaps import Sitemap

class AddressibleSitemap(Sitemap):    

    def items(self):        
        return self.model.objects.all()

    def location(self, obj):
        return obj.get_absolute_url()        

    def lastmod(self, obj):
        return obj.modified_date


from django.conf.urls import patterns, url, include

urlpatterns = patterns('',
	
	# url(r'^tags/(?P<path>[\w-]+)/$', PageTagView.as_view(), name='pages_tag'),
	# url(r'^(?P<path>[-_\/\w]*)$', PageDetail.as_view(), name="pages_page"),
	# url(r'^(?P<path>.*)$', PageDetail.as_view(), name="pages_page"),
)


from django.conf.urls import patterns, url, include

from .views import *

urlpatterns = patterns('',
	
	# url( (r'^%s/(?P<path>.*)/submitted/$'%settings.FORMS_DOMAIN), FormSubmittedView.as_view(), name='form_submitted_view'),
	# url( (r'^%s/(?P<path>.*)/(?P<pk>\d+)/$'%settings.FORMS_DOMAIN), UpdateFormEntryView.as_view(), name='form_update_view'),
	# url( (r'^%s/(?P<path>.*)/$'%settings.FORMS_DOMAIN), CreateFormEntryView.as_view(), name='form_create_view'),
	

)


import re
from bs4 import BeautifulSoup
import datetime
import feedparser
import pytz
import requests
from time import mktime

from django.core.exceptions import ImproperlyConfigured
from django.db import models

from .abstract import VersionableAtom, TitleAtom
from .content import PublishableAtom

class RSSSourceMolecule( VersionableAtom, TitleAtom ):

    #content_model = RSSEntryAtom
    
    url = models.CharField(max_length=255, null=True, blank=True)

    active = models.BooleanField(default=True)

    last_imported = models.DateTimeField(blank=True, null=True)

    logo_url = models.CharField(max_length=255, null=True, blank=True)
    logo_width = models.CharField(max_length=255, null=True, blank=True)
    logo_height = models.CharField(max_length=255, null=True, blank=True)

    regex = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        abstract = True  

    def __unicode__(self):
        return self.url

    def sync_rss(self):

        if not self.active:
            return

        response = requests.get(self.url)
        raw_xml = response.text

        feed = feedparser.parse(raw_xml)
        for entry_element in feed.entries:   
            do_import = self.import_entry(entry_element)
            if do_import:
                content_object = self.parse_entry(entry_element)

        self.last_imported = datetime.datetime.now()
        self.save()

    def get_entry_uuid(self, entry_element):
        return self.url+entry_element.id

    def import_entry(self, entry_element):
        content = entry_element.content[0].value
        if self.regex:
            prog = re.compile(self.regex, re.M|re.I)
            match = prog.match( content )

            if match:
               return True
            else:
               return False
        #OR Override in subclass

        return True

    def set_image(self, content_object, url):
        #Override in subclass
        return None
    
    
    def parse_entry(self, entry_element):   

        if not self.content_model:
            raise ImproperlyConfigured( "Model should define content_model")     

        
        synopsis = None
        if 'description' in entry_element:
            synopsis = entry_element.description
        elif 'summary' in entry_element:
            synopsis = entry_element.summary

        content = entry_element.content[0].value

        utc=pytz.UTC
        if entry_element.published_parsed:
            date = utc.localize(datetime.datetime.fromtimestamp(mktime(entry_element.published_parsed)))
        else:
            date = utc.localize(datetime.datetime.now())            

        entry, created = self.content_model.objects.get_or_create(
            rss_url=self,
            uuid = self.get_entry_uuid(entry_element)
        )

        if created:
            entry.publication_status = PublishableAtom.PUBLISHED
            entry.title = entry_element.title
            entry.slug = entry_element.id
            entry.publication_date = date
            entry.synopsis = synopsis
            entry.content = content
            entry.path_override = entry_element.link  
            entry.save()

        
            model_has_image = hasattr(self.content_model, 'image')
            if model_has_image:
                image_url = None if (not model_has_image or not 'image' in entry_element) else entry_element.image['href']
                if not image_url:
                    
                    soup = BeautifulSoup(content)
                    content_images = [image["src"] for image in soup.findAll("img")]
                    if(len(content_images) > 0):
                        image_url = content_images[0]
                        self.set_image(entry, image_url)
            



        return entry



from setuptools import setup, find_packages
#this is a test
setup(name = 'django_carbon',
      description = 'Django Carbon Library',
      version = '0.14',
      url = 'https://github.com/ninapavlich/django-carbon/',
      author = 'Nina Pavlich',
      author_email='nina@ninalp.com',
      license = 'BSD',
      packages=find_packages(exclude=['ez_setup']),
      zip_safe = False,
      include_package_data=True,
      install_requires = ['setuptools', 'Django', 'boto', 'django-ace', 
      'django-appconf', 'django-ckeditor', 'django-debug-toolbar', 
      'django-extensions','django-grappelli', 'django-imagekit', 
      'django-localflavor', 'django-reversion', 'pilkit', 
      'django-storages', 'six', 'sqlparse', 'wsgiref', 'csscompressor', 'slimit'],
      classifiers=[
            'Development Status :: 1 - Planning',
            'Environment :: Web Environment',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved',
            'Operating System :: OS Independent',
            'Programming Language :: Python'
      ]
)


from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import Library

try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()





@register.assignment_tag(takes_context=True)
def get_document_folder_add_url(context, current_folder = None):
  image_model = settings.DOCUMENT_FOLDER_MODEL.split('.')
  app_label = image_model[0]
  model_name = image_model[1]
  
  url = reverse('admin:%s_%s_add'%(app_label.lower(), model_name.lower()))  

  if current_folder:
    request = context['request']
    current_url = request.get_full_path()
    url = '%s?%s'%(url, urlencode({'parent': current_folder.pk, '_redirect_to':current_url}))

  return url


@register.assignment_tag(takes_context=True)
def get_changelist_url(context, model_admin):
    return reverse('admin:%s_%s_changelist'%(model_admin.model._meta.app_label.lower(), model_admin.model._meta.object_name.lower()))



@register.assignment_tag(takes_context=True)
def get_current_folder(context, model_name):
  image_model = model_name.split('.')
  app_label = image_model[0]
  model_name = image_model[1]
  model = get_model(app_label, model_name)

  request = context.get('request')
  folder_id = request.GET.get("folder__id__exact", None)

  if folder_id is None:
    return None
  else:
    try:
      return model._default_manager.get(id=folder_id)
    except:
      return None

@register.assignment_tag(takes_context=True)
def get_child_folders(context, model_name):
  image_model = model_name.split('.')
  app_label = image_model[0]
  model_name = image_model[1]
  model = get_model(app_label, model_name)

  request = context.get('request')
  folder_id = request.GET.get("folder__id__exact", None)

  if folder_id is None:
    return model._default_manager.filter(parent=None)
  else:
    return model._default_manager.filter(parent__id=folder_id)



from django.template import Library
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.safestring import mark_safe
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()


@register.assignment_tag()
def get_user_groups():

	app_label = settings.USER_GROUP_MODEL.split('.')[0]
	object_name = settings.USER_GROUP_MODEL.split('.')[1]
	model = get_model(app_label, object_name)
	return model.objects.all()


@register.assignment_tag()
def get_user_group_by_slug(slug):

    app_label = settings.USER_GROUP_MODEL.split('.')[0]
    object_name = settings.USER_GROUP_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    try:
        return model.objects.get(slug=slug)    
    except:
        return None


@register.assignment_tag()
def get_organizations():

    app_label = settings.ORGANIZATION_MODEL.split('.')[0]
    object_name = settings.ORGANIZATION_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    return model.objects.all()

@register.assignment_tag()
def get_organization_by_slug(slug):

    app_label = settings.ORGANIZATION_MODEL.split('.')[0]
    object_name = settings.ORGANIZATION_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    try:
        return model.objects.get(slug=slug)    
    except:
        return None                


import re
import uuid
import datetime
import smtplib
import traceback

from django.db import models
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import send_mail, EmailMultiAlternatives
from django.core.urlresolvers import reverse_lazy
from django.template import Context
from django.template import Template as DjangoTemplate
from django.utils.translation import ugettext_lazy as _
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from carbon.atoms.models.abstract import VersionableAtom
from carbon.atoms.models.access import AccessKeyAtom


from carbon.atoms.models.content import *

class EmailTemplate(VersionableAtom, TitleAtom):

    help = {
        'body_template':"Template used for email body",
        'subject_template':"Template used for email subject",
        'slug':"This slug can be referenced within templates: {% extends template-slug %}"
    }


    body_template = models.TextField(_('body template'), 
        help_text=help['body_template'], null=True, blank=True)
    subject_template = models.TextField(_('subject template'), 
        help_text=help['subject_template'], null=True, blank=True)


    def send_email_message( self, recipient_email, category, message_context={}, html_base_template=None):

        category_settings = get_category_settings_for_user(recipient_email, category)
        if category_settings.can_email() == False:
            #User has opted out
            return

        receipt_model = get_model_by_label(settings.EMAIL_RECEIPT_MODEL)
        receipt = receipt_model.create_receipt(recipient_email, category)


        message_context['email'] = recipient_email
        message_context['receipt'] = receipt
        message_context['category'] = category
        message_context['subscription_settings'] = category_settings.parent
        message_context['category_subscription_settings'] = category_settings
        message_context['settings'] = settings
        message_context['site'] = Site.objects.get_current()
        message_context['site_url'] = 'https://%s'%(Site.objects.get_current()) if settings.USE_SSL else 'http://%s'%(Site.objects.get_current())


        if html_base_template==None:
            model = get_model_by_label(settings.TEMPLATE_MODEL)

            try:
                html_base_template_object = model.objects.get(slug=settings.DEFAULT_EMAIL_TEMPLATE_SLUG)
                html_base_template = DjangoTemplate(html_base_template_object.get_content())
            except:
                raise ImproperlyConfigured( "Default Email template not found. Please check your DEFAULT_EMAIL_TEMPLATE setting.")

        
        # -- Render the Subject of the Email
        subject_template= DjangoTemplate( self.subject_template )
        subject         = subject_template.render( Context( message_context ) )
        subject         = ' '.join(subject.split()) #Remove whitespace, newlines, etc

        # -- Render the Body of the Email
        body_template   = DjangoTemplate( self.body_template )
        body            = body_template.render( Context( message_context ) )
        message         = _totext( body )

        # -- Render the Template
        message_context['subject'] = subject
        message_context['body'] = body
            
        
        message_html = html_base_template.render( Context( message_context ) )

        receipt.rendered(subject, message_html)

        # -- Send Message
        try:
            _send( recipient_email, subject, message, unicode( message_html ) )
        except smtplib.SMTPException:
            print "WARNING: ERROR SENDING EMAIL MESSAGE"
            receipt.record_error( traceback.format_exc() )

    class Meta:
        abstract = True


class EmailCategory(VersionableAtom, TitleAtom, HierarchicalAtom):
    help = {
        'can_be_viewed_online':"Allow recipients to view this online. DO NOT ENABLE if you are sending any kind of private data in this email category.",
        'requires_explicit_opt_in':"Sending these emails require user to explicitely opt-in to category.",
        'can_unsubscribe': "If these emails are transactional, then subscribe/unsubscribe functionality is not needed",
        # 'can_change_frequency': "If these emails be received at a different freququency",
    }

    can_be_viewed_online = models.BooleanField(default=False, help_text=help['can_be_viewed_online'])

    requires_explicit_opt_in = models.BooleanField(default=False, help_text=help['requires_explicit_opt_in'])
    can_unsubscribe = models.BooleanField(default=True, help_text=help['can_unsubscribe'])
    # can_change_frequency = models.BooleanField(default=True, help_text=help['can_change_frequency'])


    def can_view_online(self):
        if self.can_be_viewed_online == False:
            return False

        #Make sure override has not been applied
        if self.parent:
            return self.parent.can_view_online()

        #WE MADE IT!
        return True

    class Meta:
        abstract = True
        verbose_name_plural = 'Email categories'


class EmailReceipt(VersionableAtom, AccessKeyAtom):

    recipient_email = models.CharField(_("Recipient Email"), max_length = 255)
    rendered_subject = models.TextField(_('Rendered Subject'), blank=True, null=True)    
    rendered_body = models.TextField(_('Rendered Body'), blank=True, null=True)

    viewed = models.BooleanField(default=False,)
    viewed_from_email = models.BooleanField(default=False,)
    viewed_online = models.BooleanField(default=False,)
    view_count = models.PositiveIntegerField('View Count', default=0, null = True, blank=True)
    first_viewed_date = models.DateTimeField ( _("First Viewed Date"), blank=True, null=True )
    last_viewed_date = models.DateTimeField ( _("Last Viewed Date"), blank=True, null=True )

    category = models.ForeignKey('email.EmailCategory')

    sending_error = models.BooleanField(default=False,)
    sending_error_message = models.TextField(blank=True, null=True)

    def get_record_url(self):
        try:
            return reverse('email_record_view', kwargs = {'access_key': self.access_key }) 
        except:
            return None

    def get_rendered_url(self):
        try:
            return reverse('email_rendered_view', kwargs = {'access_key': self.access_key })
        except:
            return None

    def get_view_online_url(self):
        if self.category.can_be_viewed_online:
            try:
                return reverse('email_online_view', kwargs = {'access_key': self.access_key })
            except:
                return None
        return None

    def get_absolute_url(self):
        return self.get_view_online_url()

    @property
    def notification_settings(self):
        model = get_model_by_label(settings.EMAIL_NOTIFICATION_CATEGORY_MODEL)
        settings, created = model.objects.get_or_create(category=self.category,recipient_email=self.recipient_email)
        return settings

    def record_view(self, from_email=True, from_online=False):
        if not self.first_viewed_date:
            self.first_viewed_date = datetime.datetime.now()
        
        self.last_viewed_date = datetime.datetime.now()

        self.view_count += 1
        self.viewed = True

        if from_email:
            self.viewed_from_email = True

        if from_online:
            self.viewed_online = True

        self.save()

    def record_error(self, messsage):
        self.sending_error = True
        self.sending_error_message = messsage
        self.save()

    def rendered(self, subject, message):
        self.rendered_subject = subject
        self.rendered_body = message
        self.save()

    def render_counter(self):
        site = Site.objects.get_current() 
        protocol = 'https' if settings.USE_SSL else 'http'
        return '<img src="%s://%s%s" alt="Email Counter" style="width:1px;height:1px;" />'%(protocol, site.domain, self.get_record_url())

    @classmethod
    def create_receipt(cls, recipient_email, category):
        item = cls( recipient_email=recipient_email, category=category )
        item.save()
        return item

    class Meta:
        abstract = True


class UserSubscriptionSettings(VersionableAtom, AccessKeyAtom):
    
    recipient_email = models.CharField(_("Recipient Email"), max_length = 255)

    def get_absolute_url(self):
        try:
            return reverse('email_settings_view', kwargs = {'access_key': self.access_key })
        except:
            return None

    def get_settings(self, category=None):
        model = get_model_by_label(settings.EMAIL_CATEGORY_SUBSCRIPTION_MODEL)

        if category:
            category_settings, created = model.objects.get_or_create(parent=self,category=category)
            return category_settings
            
        else:
            return model.objects.filter(parent=self)

    
    def __unicode__(self):
        return "Subscription settings for %s"%(self.recipient_email)

    class Meta:
        abstract = True
        verbose_name_plural = 'User subscription settings'

class EmailCategorySubscriptionSettings(VersionableAtom, TitleAtom):

    parent = models.ForeignKey('email.UserSubscriptionSettings')

    category = models.ForeignKey('email.EmailCategory')

    DEFAULT = 'default'
    UNSUBSCRIBED = 'unsubscribed'
    SUBSCRIBED = 'subscribed'
    NOTIFICATION_STATUS_CHOICES = (
        (DEFAULT, _("Default")),
        (UNSUBSCRIBED, _("Unsubscribed")),
        (SUBSCRIBED, _("Subscribed")),
    )

    status = models.CharField(choices=NOTIFICATION_STATUS_CHOICES, 
        default=DEFAULT, max_length = 255)


    
    def can_email(self):
        if self.category.requires_explicit_opt_in:
            if self.status != EmailCategorySubscriptionSettings.SUBSCRIBED:
                return False
        elif self.category.can_unsubscribe:
            if self.status == EmailCategorySubscriptionSettings.UNSUBSCRIBED:
                return False

        
        #WE MADE IT!
        return True

    def save(self, *args, **kwargs):

        self.title = "Subscription settings for %s - %s"%(self.parent.recipient_email, self.category.title)           
        super(EmailCategorySubscriptionSettings, self).save(*args, **kwargs)
    
    class Meta:
        abstract = True
        verbose_name_plural = 'Email Category subscription settings'



def get_settings_for_user(recipient_email, category=None):
    notification_settings_model = get_model_by_label(settings.EMAIL_SUBSCRIPTION_MODEL)
    notification_settings, created = notification_settings_model.objects.get_or_create(recipient_email=recipient_email)

    return notification_settings

def get_category_settings_for_user(recipient_email, category=None):
    notification_settings_model = get_model_by_label(settings.EMAIL_SUBSCRIPTION_MODEL)
    notification_settings, created = notification_settings_model.objects.get_or_create(recipient_email=recipient_email)

    if category:
        return notification_settings.get_settings(category)
    else:
        return notification_settings.get_settings()       


def get_model_by_label(label):
    app_label = label.split('.')[0]
    object_name = label.split('.')[1]
    return get_model(app_label, object_name)

def _totext( data ):
    """Strips HTML from Email for Text Only"""
    p = re.compile(r'<.*?>')
    return p.sub('', data)

def _get_formatted_sender():
    try:
        return "%s <%s>" % ( settings.DEFAULT_FROM_EMAIL_NAME, settings.DEFAULT_FROM_EMAIL)
    except:
        raise ImproperlyConfigured( "The following variables must be defined in settings in order to use the emailer: DEFAULT_FROM_EMAIL_NAME, DEFAULT_FROM_EMAIL" )

def _get_formatted_recipient( recipient_email ):
    """Ensure Email Address is Returned in a List"""
    if isinstance( recipient_email, str ):
        return [ recipient_email ]
    elif isinstance( recipient_email, unicode ):
        return [ recipient_email ]
    else:
        return recipient_email

def _send( recipient_email, subject, message, message_html = None ):
    
    # -- Create Message
    msg = EmailMultiAlternatives( subject, message, _get_formatted_sender(), _get_formatted_recipient( recipient_email ) )

    # -- Attach HTML Message
    if message_html:
        msg.attach_alternative( message_html, "text/html; charset=UTF-8" )

    # -- Send Message
    msg.send()






from django.contrib import admin

from django.core.urlresolvers import reverse_lazy

from carbon.atoms.admin.taxonomy import *
from carbon.atoms.admin.media import *

# from .models import *


from django_batch_uploader.admin import BaseBatchUploadAdmin

class MediaTagAdmin(BaseSimplifiedTagAdmin):
    pass

class MediaFolderAdmin(BaseSimplifiedCategoryAdmin):
    pass




class ImageAdmin(BaseImageAdmin, BaseBatchUploadAdmin):

    
    extra_urls = [
        {'url':reverse_lazy('admin_image_batch_view'), 'title':'Batch Upload Images'}
    ]
    
    


class SecureImageAdmin(BaseImageAdmin, BaseBatchUploadAdmin):

    # extra_urls = [
    #     {'url':reverse_lazy('admin_secureimage_batch_view'), 'title':'Batch Upload Secure Images'}
    # ]
    pass
    



class MediaAdmin(BaseBatchUploadAdmin, BaseMediaAdmin):

    extra_urls = [
        {'url':reverse_lazy('admin_document_batch_view'), 'title':'Batch Upload Media'}
    ]
    

class SecureMediaAdmin(BaseBatchUploadAdmin, BaseSecureMediaAdmin):
    
    # extra_urls = [
    #     {'url':reverse_lazy('admin_securedocument_batch_view'), 'title':'Batch Upload Secure Media'}
    # ]
    pass



# admin.site.register(Image, ImageAdmin)
# admin.site.register(SecureImage, SecureImageAdmin)    
# admin.site.register(Media, MediaAdmin)    
# admin.site.register(SecureMedia, SecureMediaAdmin)    


from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.utils.decorators import method_decorator
from django.views.generic import ListView, DetailView
from django.views.generic.base import TemplateView
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from django_batch_uploader.views import AdminBatchUploadView

from .models import *

class BasePickerView(ListView):
    
    
    paginate_by = 48
    
    def get_queryset(self):
        
        if self.request.user and self.request.user.is_authenticated() and self.request.user.is_staff:
            
            if 'q' in self.request.GET and self.request.GET['q']!= '':
                query = self.request.GET['q']
                queryset = self.model.objects.filter(
                    Q(title__icontains=query) |
                    Q(caption__icontains=query) | 
                    Q(credit__icontains=query) | 
                    Q(admin_note__icontains=query) ).order_by('-id')    
            else:
                queryset = self.model.objects.all().order_by('-id')

            return queryset

        else:

            raise PermissionDenied

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(BasePickerView, self).dispatch(*args, **kwargs)

class ImagePickerView(BasePickerView):
    template_name = "admin/media/image_picker.html"
    model = get_model(settings.IMAGE_MODEL.split('.')[0], settings.IMAGE_MODEL.split('.')[1])

class ImagePickerFolderView(BasePickerView):
    template_name = "admin/media/image_picker_folders.html"
    model = get_model(settings.IMAGE_MODEL.split('.')[0], settings.IMAGE_MODEL.split('.')[1])    

    def get_folder_qs(self, request, folder_id=None):
        if folder_id is None:
          folder_id = request.POST.get('folder__id__exact', None) or request.POST.get('folder_id', None) or request.POST.get('folder', None)

        if folder_id is None:
            folder_id = request.GET.get('folder__id__exact', None) or request.GET.get('folder_id', None) or request.GET.get('folder', None)

        if folder_id=="None":
            folder_id=None

        return folder_id

    def get_folder(self, request, folder_id=None):
        
        folder_id = self.get_folder_qs(request, folder_id)
        if folder_id and int(folder_id) >= 0:
            return self.folder_model.objects.get(pk=folder_id)
        return None

    def ignore_folders(self, request):
        has_query = request.GET.get('q', None) != None
        folder_id = self.get_folder_qs(request)
        if has_query and not folder_id:
            return True

        
        if folder_id and int(folder_id) < 0:
            return True

        return False

    def get_folders(self, current_folder=None):
        if current_folder:
            return self.folder_model.objects.filter(parent=current_folder)
        else:
            return self.folder_model.objects.filter(parent=None)

    def get_all_folders(self):
        return self.folder_model.objects.all()

    def get_queryset(self):
        qs = super(ImagePickerFolderView, self).get_queryset()
        
        ignore_folders = self.ignore_folders(self.request)
        folder = self.get_folder(self.request)

        if folder is None and ignore_folders is False:
            qs = qs.filter(folder=None)
        elif folder:
            qs = qs.filter(folder=folder)

        return qs

    def get_context_data(self, **kwargs):
        context = super(ImagePickerFolderView, self).get_context_data(**kwargs)
        request = self.request

        current_folder = self.get_folder(request)
        context['folders'] = self.get_folders(current_folder)
        context['current_folder'] = current_folder
        context['ignore_folders'] = self.ignore_folders(request)
        context['top_url'] = self.request.path
        return context

class ImageBatchView(AdminBatchUploadView):      

    # model = Image

    #Media file name
    media_file_name = 'image'

    #Which fields can be applied in bulk?
    default_fields = ['credit', 'tags']

    #Which fields can be applied individually?
    detail_fields = ['title', 'alt', 'caption', 'use_png']

    default_values = {}

class SecureImageBatchView(ImageBatchView):      
    
    # model = SecureImage
    pass


class DocumentBatchView(AdminBatchUploadView):      
    
    # model = Document

    #Media file name
    media_file_name = 'file'

    #Which fields can be applied in bulk?
    default_fields = ['credit', 'tags']

    #Which fields can be applied individually?
    detail_fields = ['title',  'caption']

    default_values = {}

class SecureDocumentBatchView(DocumentBatchView):      
    
    # model = SecureDocument
    pass




from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from carbon.atoms.admin.content import *
from carbon.atoms.admin.taxonomy import *

from reversion.admin import VersionAdmin

from .models import *
# from .forms import *

from django_inline_wrestler.admin import TabularInlineOrderable


class FormEntryStatusAdmin(BaseTagAdmin):
    pass

class FormEntryTagAdmin(BaseTagAdmin):
    pass

class FormFieldInline(admin.StackedInline):
    #model = FormField
    # form = FormFieldInlineAdminForm
  
    ordering = ("order",)
    sortable_field_name = 'order'
    core_fields = (
        ('type','order'),
        ('title', 'slug'),
        ('secondary_label','help_text',),
        ('placeholder_text','default'),
        ('is_required', 'hide',),
        'error_message',
        'choices',
    )

    additional_content = (              
        'extra_css_classes',
        'third_party_id',
        # ('icon_left', 'icon_right'),
        # ('inset_text_left', 'inset_text_right'),
        'content',
    )
    validation_options = (
        ('is_digits','is_alphanumeric'),
        ('min_length','max_length'),
        # ('step_interval'),
        ('min_value','max_value'),
        ('min_words','max_words'),
        

        ('min_width', 'max_width'),
        ('min_height', 'max_height'),
        ('min_size', 'max_size'),

        # ('min_check','max_check'),
        ('min_date','min_datetime'),
        ('max_date', 'max_datetime'),
        
        
        ('equal_to', 'equal_to_error_message'),
        ('pattern', 'pattern_error_message')
    )


    fieldsets = (
        ("Basic Properties", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Additional Content", {
            'fields': additional_content,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Advanced Validation Options", {
            'fields': validation_options,
            'classes': ( 'grp-collapse grp-closed', )
        }),
    ) 
    classes = ('grp-collapse grp-open',)
    readonly_fields = ('slug',)
    # prepopulated_fields = {"slug": ("title",)}
    extra = 0  
    

class FormAdmin(VersionAdmin, BaseContentAdmin):
    core_fields = (
        ('title','slug'),       
    )
    form_behavior = (
        'redirect_url_on_submission',
        'form_action',
        'required_logged_in_user',
        'is_editable',      
        
        'extra_css_classes',
        'third_party_id'
    )
    form_messages_and_abels = (
        'submit_label',
        'form_create_message',
        'form_update_message',
        'form_error_message',
    )
    email_email_fields = (
        'email_admin_on_submission',
        'email_admin_override',
        'email_admin_on_submission_category',
        'email_admin_on_submission_template',
    
        'email_user_on_submission',
        'email_user_field_slug',
        'email_user_on_submission_template',
        'email_user_on_submission_category'
    )
    

    additional_content_fields = (
        ('publication_status'),
        ('image_preview','image'),
        'template',
        'synopsis',
        'content',
        ('submit_template'),
        ('submission_content'),
    )

    autocomplete_lookup_fields = {
        'fk': ('image', 'published_by', 'template', 'submit_template', 
            'email_admin_on_submission_template', 'email_user_on_submission_template',
            'email_admin_on_submission_category', 'email_user_on_submission_category'),
        'm2m': ()
    }
    raw_id_fields = ( 'image', 'published_by', 'template', 'submit_template', 
        'email_admin_on_submission_template', 'email_user_on_submission_template',
        'email_admin_on_submission_category', 'email_user_on_submission_category')

    publication_fields = BaseContentAdmin.publication_fields
   
    path_fields = BaseContentAdmin.path_fields

    seo_fields = BaseContentAdmin.seo_fields

    social_fields = BaseContentAdmin.social_fields
    
    meta_fields = BaseVersionableAdmin.meta_fields
    


    fieldsets = (
        ("Form Title", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Form Behavior", {
            'fields': form_behavior,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Form Messages & Labels", {
            'fields': form_messages_and_abels,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Form Email Behavior", {
            'fields': email_email_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Additional Content  (For Standalone Forms only)", {
            'fields': additional_content_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Path (For Standalone Forms only)", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication (For Standalone Forms only)", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        
        ("Search Engine Optimization (For Standalone Forms only)", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration (For Standalone Forms only)", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )


class FormEntryAdmin(VersionAdmin, BaseVersionableAdmin):

    def mark_new(modeladmin, request, queryset):
        queryset.update(status=FormEntry.NEW)
    mark_new.short_description = "Mark selected entries as new"

    def mark_read(modeladmin, request, queryset):
        queryset.update(status=FormEntry.READ)
    mark_read.short_description = "Mark selected entries as read"

    def mark_replied(modeladmin, request, queryset):
        queryset.update(status=FormEntry.REPLIED)
    mark_replied.short_description = "Mark selected entries as replied"

    def mark_archived(modeladmin, request, queryset):
        queryset.update(status=FormEntry.ARCHIVED)
    mark_archived.short_description = "Archive selected entries"
    
    actions = [mark_new, mark_read, mark_replied, mark_archived]

    list_display = ( "form_schema", "pk", "created_date",  "admin_note",'status',)
    list_filter = ("form_schema", "created_date", 'status', 'tags', "created_by", "modified_by",)

    autocomplete_lookup_fields = {
        'fk': ('form_schema',),
        'm2m': ('tags',)
    }
    raw_id_fields = ( 'form_schema','tags')

    core_fields = (
        ('form_schema'),
        ('status'),
        ('tags')
    )
    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Form", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-open', )
        })
    )

    

class FieldEntryAdmin(VersionAdmin, BaseVersionableAdmin):

    autocomplete_lookup_fields = {
        'fk': ('form_entry', 'form_field'),
    }
    raw_id_fields = ('form_entry', 'form_field',)

    core_fields = (
        'form_field',
        ('form_entry',),
        ('value')
    )
    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Form", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )



class FieldEntryInline(admin.TabularInline):
    #model = FieldEntry    

    def rendered_value(self, obj):
        return mark_safe(obj.get_rendered_value())

    


    readonly_fields = ('form_field', 'rendered_value')
    fields = ('form_field', 'value', 'rendered_value')
    extra = 0



from django.conf import settings
from django.template import Library, Context, Template, TemplateSyntaxError, Variable, Node
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

register = Library()


@register.assignment_tag(takes_context=True)
def get_email_category_by_id(context, id):

    app_label = settings.EMAIL_CATEGORY_MODEL.split('.')[0]
    object_name = settings.EMAIL_CATEGORY_MODEL.split('.')[1]
    model = get_model(app_label, object_name)
    try:
        return model.objects.get(pk=id)
    except:
        return None


from django.contrib import messages
from django.dispatch import receiver
from django.http import Http404
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.generic import DetailView

from carbon.atoms.views.abstract import *
from carbon.atoms.views.content import *



from .models import *
from .forms import *


class PageDetail(NonAdminCachableView, PublishableView, AddressibleView, DetailView):

	# model = Page
	pass


class SiblingPageDetail(NonAdminCachableView, PublishableView, AddressibleView, HasSiblingsView, DetailView):

	# model = Page
	pass	
	




class PageTagView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, DetailView):

	# model = PageTag
	pass

		
class PageBlockView(object):

	def get_context_data(self, **kwargs):
		
		ctx = super(PageBlockView, self).get_context_data(**kwargs)

		#PAGE CONTENT BLOCKS
		pageblocks = self.get_page_content_blocks()
		for pageblock in pageblocks:
			ctx[pageblock.slug] = pageblock

		ctx['page_blocks'] = pageblocks
	   
		return ctx

	def get_page_content_blocks(self):

		return self.object.get_page_content_blocks()





"""
Wrapper for loading templates from the filesystem.
"""

import io

from django.core.exceptions import SuspiciousFileOperation
from django.template import TemplateDoesNotExist
from django.utils._os import safe_join
from django.template import Template as DjangoTemplate
try:
    from django.template.loaders.base import Loader as BaseLoader
except ImportError:  # Django < 1.9
    from django.template.loader import BaseLoader


from .models import Template

from carbon.utils.template import get_template_by_pk_or_slug

class DBTemplateLoader(BaseLoader):
    is_usable = True

    
    def load_template(self, template_name, template_dirs=None):
        # print 'get template: %s'%(template_name)
        template = get_template_by_pk_or_slug(template_name)
        if template:
            return DjangoTemplate(template.get_content()), template_name
            
        raise TemplateDoesNotExist("No existe template %s"%(template_name))


    def load_template_source(self, template_name, template_dirs=None):
        # print 'load template source %s'%(template_name)
        template = self.load_template(template_name, template_dirs=None)
        if template:
            return template.get_content(), template_name

        raise TemplateDoesNotExist("No existe template %s"%(template_name))
    load_template_source.is_usable = True


    # #HACK -- argument incompatibility
    # def __init__(self, *args):
    #     kwargs = {}
    #     super(DBTemplateLoader, self).__init__(self, *args, **kwargs) 

    
